@@ -178,7 +178,7 @@ def query(params=None):
     if request.status_code != 200:
         raise SaltCloudSystemExit(
             'An error occurred while querying QingCloud. HTTP Code: {0}  '
-            'Error: {1!r}'.format(
+            'Error: \'{1}\''.format(
                 request.status_code,
                 request.text
             )
@@ -248,7 +248,7 @@ def _get_location(vm_=None):
         return vm_location
 
     raise SaltCloudNotFound(
-        'The specified location, {0!r}, could not be found.'.format(
+        'The specified location, \'{0}\', could not be found.'.format(
             vm_location
         )
     )
@@ -319,7 +319,7 @@ def _get_image(vm_):
         return vm_image
 
     raise SaltCloudNotFound(
-        'The specified image, {0!r}, could not be found.'.format(vm_image)
+        'The specified image, \'{0}\', could not be found.'.format(vm_image)
     )
 
 
@@ -440,7 +440,7 @@ def _get_size(vm_):
         return vm_size
 
     raise SaltCloudNotFound(
-        'The specified size, {0!r}, could not be found.'.format(vm_size)
+        'The specified size, \'{0}\', could not be found.'.format(vm_size)
     )
 
 
@@ -517,7 +517,7 @@ def list_nodes_full(call=None):
         provider = comps[0]
 
     __opts__['update_cachedir'] = True
-    salt.utils.cloud.cache_node_list(result, provider, __opts__)
+    __utils__['cloud.cache_node_list'](result, provider, __opts__)
 
     return result
 
@@ -623,7 +623,7 @@ def show_instance(instance_id, call=None, kwargs=None):
 
     if items['total_count'] == 0:
         raise SaltCloudNotFound(
-            'The specified instance, {0!r}, could not be found.'.format(instance_id)
+            'The specified instance, \'{0}\', could not be found.'.format(instance_id)
         )
 
     full_node = items['instance_set'][0]
@@ -671,7 +671,7 @@ def create(vm_):
     if 'provider' in vm_:
         vm_['driver'] = vm_.pop('provider')
 
-    salt.utils.cloud.fire_event(
+    __utils__['cloud.fire_event'](
         'event',
         'starting create',
         'salt/cloud/{0}/creating'.format(vm_['name']),
@@ -697,7 +697,7 @@ def create(vm_):
         'login_keypair': vm_['login_keypair'],
     }
 
-    salt.utils.cloud.fire_event(
+    __utils__['cloud.fire_event'](
         'event',
         'requesting instance',
         'salt/cloud/{0}/requesting'.format(vm_['name']),
@@ -735,17 +735,17 @@ def create(vm_):
     vm_['ssh_host'] = private_ip
 
     # The instance is booted and accessible, let's Salt it!
-    salt.utils.cloud.bootstrap(vm_, __opts__)
+    __utils__['cloud.bootstrap'](vm_, __opts__)
 
-    log.info('Created Cloud VM {0[name]!r}'.format(vm_))
+    log.info('Created Cloud VM \'{0[name]}\''.format(vm_))
 
     log.debug(
-        '{0[name]!r} VM creation details:\n{1}'.format(
+        '\'{0[name]}\' VM creation details:\n{1}'.format(
             vm_, pprint.pformat(data)
         )
     )
 
-    salt.utils.cloud.fire_event(
+    __utils__['cloud.fire_event'](
         'event',
         'created instance',
         'salt/cloud/{0}/created'.format(vm_['name']),
@@ -879,7 +879,7 @@ def destroy(instance_id, call=None):
     instance_data = show_instance(instance_id, call='action')
     name = instance_data['instance_name']
 
-    salt.utils.cloud.fire_event(
+    __utils__['cloud.fire_event'](
         'event',
         'destroying instance',
         'salt/cloud/{0}/destroying'.format(name),
@@ -894,7 +894,7 @@ def destroy(instance_id, call=None):
     }
     result = query(params)
 
-    salt.utils.cloud.fire_event(
+    __utils__['cloud.fire_event'](
         'event',
         'destroyed instance',
         'salt/cloud/{0}/destroyed'.format(name),
