@@ -247,6 +247,22 @@ command sent to minions as well as a runner function on the master::
 .. |200| replace:: success
 .. |401| replace:: authentication required
 .. |406| replace:: requested Content-Type not available
+
+A Note About Curl
+=================
+
+When sending passwords and data that might need to be urlencoded, you must set
+the ``-d`` flag to indicate the content type, and the ``--data-urlencode`` flag
+to urlencode the input.
+
+.. code-block:: bash
+
+    curl -ksi http://localhost:8000/login \\
+    -H "Accept: application/json" \\
+    -d username='myapiuser' \\
+    --data-urlencode password='1234+' \\
+    -d eauth='pam'
+
 '''
 # We need a custom pylintrc here...
 # pylint: disable=W0212,E1101,C0103,R0201,W0221,W0613
@@ -266,7 +282,6 @@ from multiprocessing import Process, Pipe
 # Import third-party libs
 # pylint: disable=import-error
 import cherrypy
-from cherrypy.lib import cpstats
 import yaml
 import salt.ext.six as six
 # pylint: enable=import-error
@@ -1528,7 +1543,7 @@ class Login(LowDataAdapter):
             perms = eauth.get(token['name'], [])
             perms.extend(eauth.get('*', []))
 
-            if 'groups' in token and token['groups'] is not False:
+            if 'groups' in token and token['groups']:
                 user_groups = set(token['groups'])
                 eauth_groups = set([i.rstrip('%') for i in eauth.keys() if i.endswith('%')])
 
@@ -1536,13 +1551,12 @@ class Login(LowDataAdapter):
                     perms.extend(eauth['{0}%'.format(group)])
 
             if not perms:
-                raise ValueError("Eauth permission list not found.")
-        except (AttributeError, IndexError, KeyError, ValueError):
+                logger.debug("Eauth permission list not found.")
+        except Exception:
             logger.debug("Configuration for external_auth malformed for "
                 "eauth '{0}', and user '{1}'."
                 .format(token.get('eauth'), token.get('name')), exc_info=True)
-            raise cherrypy.HTTPError(500,
-                'Configuration for external_auth could not be read.')
+            perms = None
 
         return {'return': [{
             'token': cherrypy.session.id,
@@ -1550,7 +1564,7 @@ class Login(LowDataAdapter):
             'start': token['start'],
             'user': token['name'],
             'eauth': token['eauth'],
-            'perms': perms,
+            'perms': perms or {},
         }]}
 
 
@@ -2266,6 +2280,13 @@ class Stats(object):
             :status 406: |406|
         '''
         if hasattr(logging, 'statistics'):
+            # Late import
+            try:
+                from cherrypy.lib import cpstats
+            except ImportError:
+                logger.error('Import of cherrypy.cpstats failed. Possible '
+                        'upstream bug here: https://github.com/cherrypy/cherrypy/issues/1444')
+                return {}
             return cpstats.extrapolate_statistics(logging.statistics)
 
         return {}
