@@ -145,6 +145,8 @@ class StateModuleTest(integration.ModuleCase,
         ret = self.run_function('state.sls', mods='testappend.step-2')
         self.assertSaltTrueReturn(ret)
 
+        with salt.utils.fopen(testfile, 'r') as fp_:
+            contents = fp_.read()
         self.assertMultiLineEqual(textwrap.dedent('''\
             # set variable identifying the chroot you work in (used in the prompt below)
             if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
@@ -155,7 +157,7 @@ class StateModuleTest(integration.ModuleCase,
             if [ -f /etc/bash_completion ] && ! shopt -oq posix; then
                 . /etc/bash_completion
             fi
-            '''), salt.utils.fopen(testfile, 'r').read())
+            '''), contents)
 
         # Re-append switching order
         ret = self.run_function('state.sls', mods='testappend.step-2')
@@ -164,6 +166,8 @@ class StateModuleTest(integration.ModuleCase,
         ret = self.run_function('state.sls', mods='testappend.step-1')
         self.assertSaltTrueReturn(ret)
 
+        with salt.utils.fopen(testfile, 'r') as fp_:
+            contents = fp_.read()
         self.assertMultiLineEqual(textwrap.dedent('''\
             # set variable identifying the chroot you work in (used in the prompt below)
             if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
@@ -174,7 +178,7 @@ class StateModuleTest(integration.ModuleCase,
             if [ -f /etc/bash_completion ] && ! shopt -oq posix; then
                 . /etc/bash_completion
             fi
-            '''), salt.utils.fopen(testfile, 'r').read())
+            '''), contents)
 
     def test_issue_1876_syntax_error(self):
         '''
@@ -199,7 +203,7 @@ class StateModuleTest(integration.ModuleCase,
         )
 
     def test_issue_1879_too_simple_contains_check(self):
-        contents = textwrap.dedent('''\
+        expected = textwrap.dedent('''\
             # set variable identifying the chroot you work in (used in the prompt below)
             if [ -z "$debian_chroot" ] && [ -r /etc/debian_chroot ]; then
                 debian_chroot=$(cat /etc/debian_chroot)
@@ -232,10 +236,9 @@ class StateModuleTest(integration.ModuleCase,
 
         # Does it match?
         try:
-            self.assertMultiLineEqual(
-                contents,
-                salt.utils.fopen(testfile, 'r').read()
-            )
+            with salt.utils.fopen(testfile, 'r') as fp_:
+                contents = fp_.read()
+            self.assertMultiLineEqual(expected, contents)
             # Make sure we don't re-append existing text
             ret = self.run_function(
                 'state.sls', mods='issue-1879.step-1', timeout=120
@@ -246,10 +249,10 @@ class StateModuleTest(integration.ModuleCase,
                 'state.sls', mods='issue-1879.step-2', timeout=120
             )
             self.assertSaltTrueReturn(ret)
-            self.assertMultiLineEqual(
-                contents,
-                salt.utils.fopen(testfile, 'r').read()
-            )
+
+            with salt.utils.fopen(testfile, 'r') as fp_:
+                contents = fp_.read()
+            self.assertMultiLineEqual(expected, contents)
         except Exception:
             if os.path.exists(testfile):
                 shutil.copy(testfile, testfile + '.bak')
@@ -321,7 +324,8 @@ class StateModuleTest(integration.ModuleCase,
             'files', 'file', 'base', 'issue-2068-template-str-no-dot.sls'
         )
 
-        template = salt.utils.fopen(template_path, 'r').read()
+        with salt.utils.fopen(template_path, 'r') as fp_:
+            template = fp_.read()
         try:
             ret = self.run_function(
                 'state.template_str', [template], timeout=120
@@ -362,7 +366,8 @@ class StateModuleTest(integration.ModuleCase,
             'files', 'file', 'base', 'issue-2068-template-str.sls'
         )
 
-        template = salt.utils.fopen(template_path, 'r').read()
+        with salt.utils.fopen(template_path, 'r') as fp_:
+            template = fp_.read()
         try:
             ret = self.run_function(
                 'state.template_str', [template], timeout=120
